package sun.security.krb5;

import sun.security.krb5.internal.PAData;
import sun.security.util.DerInputStream;
import sun.security.krb5.internal.Krb5;
import sun.security.krb5.internal.KrbApErrException;
import sun.security.krb5.internal.KDCRep;
import sun.security.krb5.internal.KDCReq;

abstract class KrbKdcRep
{
    KrbKdcRep() {
    }
    
    static void check(boolean b, KDCReq kdcReq, KDCRep kdcRep, EncryptionKey encryptionKey) throws KrbApErrException {
        if (b && !kdcReq.reqBody.cname.equals(kdcRep.cname) && ((!kdcReq.reqBody.kdcOptions.get(15) && kdcReq.reqBody.cname.getNameType() != 10) || !kdcRep.encKDCRepPart.flags.get(15))) {
            kdcRep.encKDCRepPart.key.destroy();
            throw new KrbApErrException(41);
        }
        if (!kdcReq.reqBody.sname.equals(kdcRep.encKDCRepPart.sname)) {
            String[] nameStrings = kdcRep.encKDCRepPart.sname.getNameStrings();
            if (b || !kdcReq.reqBody.kdcOptions.get(15) || nameStrings == null || nameStrings.length != 2 || !nameStrings[0].equals("krbtgt") || !kdcRep.encKDCRepPart.sname.getRealmString().equals(kdcReq.reqBody.sname.getRealmString())) {
                kdcRep.encKDCRepPart.key.destroy();
                throw new KrbApErrException(41);
            }
        }
        if (kdcReq.reqBody.getNonce() != kdcRep.encKDCRepPart.nonce) {
            kdcRep.encKDCRepPart.key.destroy();
            throw new KrbApErrException(41);
        }
        if (kdcReq.reqBody.addresses != null && kdcRep.encKDCRepPart.caddr != null && !kdcReq.reqBody.addresses.equals(kdcRep.encKDCRepPart.caddr)) {
            kdcRep.encKDCRepPart.key.destroy();
            throw new KrbApErrException(41);
        }
        int i = 2;
        while (i < 6) {
            if (kdcReq.reqBody.kdcOptions.get(i) != kdcRep.encKDCRepPart.flags.get(i)) {
                if (Krb5.DEBUG) {
                    System.out.println("> KrbKdcRep.check: at #" + i + ". request for " + kdcReq.reqBody.kdcOptions.get(i) + ", received " + kdcRep.encKDCRepPart.flags.get(i));
                }
                throw new KrbApErrException(41);
            }
            else {
                ++i;
            }
        }
        if (kdcReq.reqBody.kdcOptions.get(8) && !kdcRep.encKDCRepPart.flags.get(8)) {
            throw new KrbApErrException(41);
        }
        if ((kdcReq.reqBody.from == null || kdcReq.reqBody.from.isZero()) && kdcRep.encKDCRepPart.starttime != null && !kdcRep.encKDCRepPart.starttime.inClockSkew()) {
            kdcRep.encKDCRepPart.key.destroy();
            throw new KrbApErrException(37);
        }
        if (kdcReq.reqBody.from != null && !kdcReq.reqBody.from.isZero() && kdcRep.encKDCRepPart.starttime != null && !kdcReq.reqBody.from.equals(kdcRep.encKDCRepPart.starttime)) {
            kdcRep.encKDCRepPart.key.destroy();
            throw new KrbApErrException(41);
        }
        if (!kdcReq.reqBody.till.isZero() && kdcRep.encKDCRepPart.endtime.greaterThan(kdcReq.reqBody.till)) {
            kdcRep.encKDCRepPart.key.destroy();
            throw new KrbApErrException(41);
        }
        if (kdcReq.reqBody.kdcOptions.get(8) && kdcReq.reqBody.rtime != null && !kdcReq.reqBody.rtime.isZero() && (kdcRep.encKDCRepPart.renewTill == null || kdcRep.encKDCRepPart.renewTill.greaterThan(kdcReq.reqBody.rtime))) {
            kdcRep.encKDCRepPart.key.destroy();
            throw new KrbApErrException(41);
        }
        if (kdcReq.reqBody.kdcOptions.get(27) && kdcRep.encKDCRepPart.flags.get(8) && !kdcReq.reqBody.till.isZero() && (kdcRep.encKDCRepPart.renewTill == null || kdcRep.encKDCRepPart.renewTill.greaterThan(kdcReq.reqBody.till))) {
            kdcRep.encKDCRepPart.key.destroy();
            throw new KrbApErrException(41);
        }
        if (kdcRep.encKDCRepPart.flags.get(15)) {
            boolean n = false;
            boolean verifyAnyChecksum = false;
            if (kdcReq.pAData != null) {
                PAData[] paData = kdcReq.pAData;
                int length = paData.length;
                int j = 0;
                while (j < length) {
                    if (paData[j].getType() == 149) {
                        n = true;
                        break;
                    }
                    ++j;
                }
            }
            if (kdcRep.encKDCRepPart.pAData != null) {
                PAData[] paData2 = kdcRep.encKDCRepPart.pAData;
                int length2 = paData2.length;
                int k = 0;
                while (k < length2) {
                    PAData paData3 = paData2[k];
                    if (paData3.getType() == 149) {
                        try {
                            verifyAnyChecksum = new Checksum(new DerInputStream(paData3.getValue()).getDerValue()).verifyAnyChecksum(kdcReq.asn1Encode(), encryptionKey, 56);
                        }
                        catch (Exception ex) {
                            if (Krb5.DEBUG) {
                                ex.printStackTrace();
                            }
                        }
                        break;
                    }
                    ++k;
                }
            }
            if (n && verifyAnyChecksum == 0) {
                throw new KrbApErrException(41);
            }
        }
        return;
    }
}
